import PropTypes from 'prop-types';
import React, { useState, useEffect, useRef } from 'react';
import { useMount } from '@s-ui/react-hooks';
import SuggestsList from './suggests-list';
import cx from 'classnames';
var DELTA_MOVE = 1;
var UP = 'ArrowUp';
var DOWN = 'ArrowDown';
var ENTER = 'Enter';
var ESCAPE = 'Escape';
export default function FormAutocompleted(props) {
  var _props$initialValue = props.initialValue,
      initialValue = _props$initialValue === void 0 ? '' : _props$initialValue,
      suggests = props.suggests,
      handleChange = props.handleChange,
      handleSubmit = props.handleSubmit,
      handleSelect = props.handleSelect,
      handleFocus = props.handleFocus,
      handleBlur = props.handleBlur,
      _props$selectFirstByD = props.selectFirstByDefault,
      selectFirstByDefault = _props$selectFirstByD === void 0 ? true : _props$selectFirstByD,
      _props$focus = props.focus,
      focus = _props$focus === void 0 ? false : _props$focus,
      _props$closeOnOutside = props.closeOnOutsideClick,
      closeOnOutsideClick = _props$closeOnOutside === void 0 ? false : _props$closeOnOutside,
      placeholder = props.placeholder,
      submit = props.submit,
      collapsed = props.collapsed;
  var inputEl = useRef(null);
  var submitEl = useRef(null);
  var suggestList = useRef(null);
  var excludeFromOutsideClick = [];
  var defaultPosition = selectFirstByDefault ? 0 : -1;

  var _useState = useState(defaultPosition),
      active = _useState[0],
      setActive = _useState[1];

  var _useState2 = useState(initialValue),
      value = _useState2[0],
      setValue = _useState2[1];

  var _useState3 = useState(false),
      showSuggestsList = _useState3[0],
      setShowSuggestsList = _useState3[1];

  var formAutocompletedClassName = cx('sui-FormAutocompleted', {
    'is-collapsed': submit && collapsed
  });

  var _moveDown = function _moveDown() {
    var lastPosition = suggests.length - 1;
    return active === lastPosition ? active : active + DELTA_MOVE;
  };

  var _moveUp = function _moveUp() {
    return active === defaultPosition ? active : active - DELTA_MOVE;
  };

  var _upDownHandler = function _upDownHandler(event) {
    // Never go to negative values or value higher than the list length.
    var active = event.key === DOWN ? _moveDown() : _moveUp();
    setActive(active);
    event.stopPropagation();
    event.preventDefault();
  };

  var _enterHandler = function _enterHandler() {
    var suggest = suggests[active];

    if (suggest) {
      var _value = suggest.literal || suggest.content;

      setValue(_value);

      _handleSelect(suggest);
    } else {
      _handleSubmit();
    }
  };

  var _escapeHandler = function _escapeHandler() {
    setShowSuggestsList(false);
    setActive(null);
  };

  var _handleChange = function _handleChange(event) {
    var value = event.target.value;
    setValue(value);
    setActive(defaultPosition);
    handleChange(value);
  };

  var _handleSubmit = function _handleSubmit() {
    handleSubmit(value);
  };

  var _handleClear = function _handleClear() {
    _handleChange({
      target: {
        value: ''
      }
    });

    _focusInput();
  };

  var _handleSelect = function _handleSelect(suggest) {
    setValue(suggest.literal || suggest.content);
    handleSelect(suggest);
  };

  var _handleKeyDown = function _handleKeyDown(event) {
    setShowSuggestsList(true);

    switch (event.key) {
      case UP:
      case DOWN:
        _upDownHandler(event);

        break;

      case ENTER:
        _enterHandler();

        break;

      case ESCAPE:
        _escapeHandler();

        break;
    }
  };

  var _handleOutsideClick = function _handleOutsideClick(event) {
    return !excludeFromOutsideClick.includes(event.target) && setShowSuggestsList(false);
  };

  var _focusInput = function _focusInput() {
    inputEl.current.focus();
  };

  var _renderSubmitButton = function _renderSubmitButton(_ref // eslint-disable-line
  ) {
    var text = _ref.text,
        Icon = _ref.icon;
    return React.createElement("button", {
      className: "sui-FormAutocompleted-submit",
      onClick: _handleSubmit,
      ref: submitEl
    }, Icon && React.createElement(Icon, {
      svgClass: "sui-FormAutocompleted-submitIcon"
    }), text);
  };

  var _renderSuggestsList = function _renderSuggestsList() {
    return suggests && suggests.length > 0 ? React.createElement(SuggestsList, Object.assign({}, props, {
      handleSelect: _handleSelect,
      active: active,
      ref: suggestList
    })) : null;
  };

  useMount(function () {
    excludeFromOutsideClick = [inputEl, submitEl, suggestList];
    closeOnOutsideClick === true && window.addEventListener('click', _handleOutsideClick, false);
    return function () {
      window.removeEventListener('click', _handleOutsideClick, false);
    };
  });
  useEffect(function () {
    if (focus) _focusInput();
  }, [focus]);
  return React.createElement("div", {
    className: "sui-FormAutocompleted-wrap"
  }, React.createElement("div", {
    className: formAutocompletedClassName
  }, React.createElement("div", {
    className: "sui-FormAutocompleted-inputWrap"
  }, React.createElement("input", {
    ref: inputEl,
    value: value,
    placeholder: placeholder,
    className: "sui-FormAutocompleted-input",
    type: "text",
    onChange: _handleChange,
    onKeyDown: _handleKeyDown,
    onFocus: handleFocus,
    onBlur: handleBlur
  }), value && React.createElement("span", {
    className: "sui-FormAutocompleted-clear",
    onClick: _handleClear
  })), submit && _renderSubmitButton(submit)), showSuggestsList && _renderSuggestsList());
}
FormAutocompleted.displayName = 'FormAutocompleted';